//
//  NSString+Input.m
//  LYCategory
//
//  Created by Rick Luo on 11/25/13.
//  Copyright (c) 2013 Luo Yu. All rights reserved.
//

#import "NSString+Input.h"

@implementation NSString (Input)

#pragma mark EMPTY

- (BOOL)isEmpty {
	
	if (self == nil) {
		return YES;
	}
	
	if ([self isEqualToString:@""]) {
		return YES;
	}
	
	// FOR AFNETWORKING JSON PARSER
	if ([[NSString stringWithFormat:@"%@", self] isEqualToString:@"(null)"] ||
		[[NSString stringWithFormat:@"%@", self] isEqualToString:@"<null>"]) {
		return YES;
	}
	
	return NO;
}

#pragma mark SPACE

- (NSString *)trimStartSpace {
	// TODO: NOT FINISHED
	return [self stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
}

- (NSString *)trimSpace {
	return [self stringByReplacingOccurrencesOfString:@" " withString:@""];
}

#pragma mark EMAIL

- (BOOL)isEmail {
	
	// RFC 2822
	
	NSString *emailRegex =
	@"(?:[a-z0-9!#$%\\&'*+/=?\\^_`{|}~-]+(?:\\.[a-z0-9!#$%\\&'*+/=?\\^_`{|}"
	@"~-]+)*|\"(?:[\\x01-\\x08\\x0b\\x0c\\x0e-\\x1f\\x21\\x23-\\x5b\\x5d-\\"
	@"x7f]|\\\\[\\x01-\\x09\\x0b\\x0c\\x0e-\\x7f])*\")@(?:(?:[a-z0-9](?:[a-"
	@"z0-9-]*[a-z0-9])?\\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\\[(?:(?:25[0-5"
	@"]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-"
	@"9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\\x01-\\x08\\x0b\\x0c\\x0e-\\x1f\\x21"
	@"-\\x5a\\x53-\\x7f]|\\\\[\\x01-\\x09\\x0b\\x0c\\x0e-\\x7f])+)\\])";
    
	NSPredicate *emailTest = [NSPredicate predicateWithFormat:@"SELF MATCHES[c] %@", emailRegex];
	
    return [emailTest evaluateWithObject:self];
	
}

#pragma mark ID

- (BOOL)isIDNumber {
	
	if (self.length != 15 && self.length != 18) {
		return NO;
	}
	
	NSArray *provinces = @[
						   @"11",/* 北京 */ @"12",/* 天津 */
						   @"13",/* 河北 */ @"14",/* 山西 */ @"15",/* 内蒙古 */
						   @"21",/* 辽宁 */ @"22",/* 吉林 */ @"23",/* 黑龙江 */
						   @"31",/* 上海 */
						   @"32",/* 江苏 */ @"33",/* 浙江 */ @"34",/* 安徽 */
						   @"35",/* 福建 */ @"36",/* 江西 */ @"37",/* 山东 */
						   @"41",/* 河南 */ @"42",/* 湖北 */ @"43",/* 湖南 */
						   @"44",/* 广东 */ @"45",/* 广西 */ @"46",/* 海南 */
						   @"50",/* 重庆 */ @"51",/* 四川 */ @"52",/* 贵州 */
						   @"53",/* 云南 */ @"54",/* 西藏 */
						   @"61",/* 陕西 */ @"62",/* 甘肃 */ @"63",/* 青海 */
						   @"64",/* 宁夏 */ @"65",//"新疆 */
						   @"71",/* 台湾 */
						   @"81",/* 香港 */ @"82",/* 澳门 */
						   @"91",//"国外"
						   ];
	
	if ([provinces containsObject:[self substringToIndex:2]] == NO) {
		return NO;
	}
	// PASSED PROVINCE CHECK
	
	NSString *strBirth;
	if (self.length == 15) {
		strBirth = [NSString stringWithFormat:@"19%@", [self substringWithRange:NSMakeRange(6, 6)]];
	} else if (self.length == 18) {
		strBirth = [self substringWithRange:NSMakeRange(6, 8)];
	}
	
	if ([[strBirth substringWithRange:NSMakeRange(4, 2)] intValue] > 12) {
		return NO;
	}
	
	if ([[strBirth substringWithRange:NSMakeRange(6, 2)] intValue] > 31) {
		return NO;
	}
	
	if (self.length == 18) {
		
		NSMutableArray *chars = [NSMutableArray arrayWithCapacity:1];
		for (int i = 0; i < self.length; i++) {
			NSRange composedCharRange = [self rangeOfComposedCharacterSequenceAtIndex:i];
			NSString *character = [self substringWithRange:composedCharRange];
			if (character) {
				[chars addObject:character];
			}
		}
		
		NSInteger encode = ([chars[0] intValue] + [chars[10] intValue]) * 7
		+ ([chars[1] intValue] + [chars[11] intValue]) * 9
		+ ([chars[2] intValue] + [chars[12] intValue]) * 10
		+ ([chars[3] intValue] + [chars[13] intValue]) * 5
		+ ([chars[4] intValue] + [chars[14] intValue]) * 8
		+ ([chars[5] intValue] + [chars[15] intValue]) * 4
		+ ([chars[6] intValue] + [chars[16] intValue]) * 2
		+ [chars[7] intValue] * 1
		+ [chars[8] intValue] * 6
		+ [chars[9] intValue] * 3;
		
		NSString *validationString = @"10X98765432";
		NSString *validRet = [validationString substringWithRange:NSMakeRange(encode % 11, 1)];
		if ([[validRet uppercaseString] isEqualToString:[[self substringFromIndex:17] uppercaseString]] == NO) {
			return NO;
		}
	}
	
	return YES;
}

#pragma mark PHONE NUMBER

- (NSString *)phoneNumber {
	return [[self componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"/.,()-+  "]] componentsJoinedByString:@""];
}

- (BOOL)isPhoneNumber {
	
	/*
	Updated : 2017-03-01
	China Mobile：	134 135 136 137 138 139 147 150 151 152 157 158 159 178 182 183 184 187 188 198
	China Unicom：	130 131 132 145 155 156 166 171 175 176 185 186
	China Telecom：	133 149 153 173 177 180 181 189 199
	虚拟运营商:	170
	*/
	
	NSPredicate *predCMCC = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", @"^1(3[4-9]|47|5[0-27-9]|78|8[2-478]|98)\\d{8}$"];
	NSPredicate *predCU = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", @"^1(3[0-2]|45|5[56]|66|7[156]|8[56])\\d{8}$"];
	NSPredicate *predCT = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", @"^1(33|49|53|7[37]|8[019]|99)\\d{8}$"];
	NSPredicate *predV = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", @"^1(70)\\d{8}$"];
	
	return [predCMCC evaluateWithObject:self] ||
			[predCU evaluateWithObject:self] ||
			[predCT evaluateWithObject:self] ||
			[predV evaluateWithObject:self];
}

#pragma mark NUMBER

- (BOOL)isPureNumber {
	return [[NSPredicate predicateWithFormat:@"SELF MATCHES %@", @"\\d"] evaluateWithObject:self];
}

#pragma mark EMOJI

- (NSString *)replaceEmojiTextWithUnicode {
	
	NSDictionary *emojis = @{
		@"[哈哈]": @"😄",
        @"[laughing]": @"😆",
        @"[blush]": @"😊",
        @"[微笑]": @"😃",
        @"[relaxed]": @"☺",
        @"[假笑]": @"😏",
        @"[heart_eyes]": @"😍",
        @"[kissing_heart]": @"😘",
        @"[亲吻]": @"😚",
        @"[脸红]": @"😳",
        @"[relieved]": @"😥",
        @"[满足]": @"😌",
        @"[grin]": @"😁",
        @"[wink]": @"😉",
        @"[wink2]": @"😜",
        @"[stuck_out_tongue_winking_eye]": @"😜",
        @"[stuck_out_tongue_closed_eyes]": @"😝",
        @"[grinning]": @"😀",
        @"[kissing]": @"😗",
        @"[kissing_smiling_eyes]": @"😙",
        @"[stuck_out_tongue]": @"😛",
        @"[sleeping]": @"😴",
        @"[worried]": @"😟",
        @"[frowning]": @"😦",
        @"[anguished]": @"😧",
        @"[open_mouth]": @"😮",
        @"[grimacing]": @"😬",
        @"[confused]": @"😕",
        @"[hushed]": @"😯",
        @"[expressionless]": @"😑",
        @"[鄙视]": @"😒",
        @"[sweat_smile]": @"😅",
        @"[冷汗]": @"😓",
        @"[weary]": @"😩",
        @"[pensive]": @"😔",
        @"[失望]": @"😞",
        @"[confounded]": @"😖",
        @"[害怕]": @"😨",
        @"[糟糕]": @"😰",
        @"[persevere]": @"😣",
        @"[悲伤]": @"😢",
        @"[大哭]": @"😭",
        @"[joy]": @"😂",
        @"[astonished]": @"😲",
        @"[恐怖]": @"😱",
        @"[tired_face]": @"😫",
        @"[angry]": @"😠",
        @"[愤怒]": @"😡",
        @"[triumph]": @"😤",
        @"[sleepy]": @"😪",
        @"[yum]": @"😋",
        @"[mask]": @"😷",
        @"[sunglasses]": @"😎",
        @"[dizzy_face]": @"😵",
        @"[imp]": @"👿",
        @"[smiling_imp]": @"😈",
        @"[neutral_face]": @"😐",
        @"[no_mouth]": @"😶",
        @"[innocent]": @"😇",
        @"[alien]": @"👽",
        @"[黄心]": @"💛",
        @"[blue_heart]": @"💙",
        @"[purple_heart]": @"💜",
        @"[心]": @"❤",
        @"[green_heart]": @"💚",
        @"[心碎]": @"💔",
        @"[heartbeat]": @"💓",
        @"[heartpulse]": @"💗",
        @"[two_hearts]": @"💕",
        @"[revolving_hearts]": @"💞",
        @"[cupid]": @"💘",
        @"[sparkling_heart]": @"💖",
        @"[星星]": @"✨",
        @"[star]": @"⭐",
        @"[star2]": @"🌟",
        @"[dizzy]": @"💫",
        @"[boom]": @"💥",
        @"[collision]": @"💥",
        @"[anger]": @"💢",
        @"[exclamation]": @"❗",
        @"[question]": @"❓",
        @"[grey_exclamation]": @"❕",
        @"[grey_question]": @"❔",
        @"[zzz]": @"💤",
        @"[dash]": @"💨",
        @"[sweat_drops]": @"💦",
        @"[notes]": @"🎶",
        @"[musical_note]": @"🎵",
        @"[火焰]": @"🔥",
        @"[hankey]": @"💩",
        @"[poop]": @"💩",
        @"[shit]": @"💩",
        @"[点赞]": @"👍",
        @"[+1]": @"👍",
        @"[thumbsdown]": @"👎",
        @"[-1]": @"👎",
        @"[ok_hand]": @"👌",
        @"[punch]": @"👊",
        @"[facepunch]": @"👊",
        @"[fist]": @"✊",
        @"[v]": @"✌",
        @"[挥手]": @"👋",
        @"[hand]": @"✋",
        @"[open_hands]": @"👐",
        @"[point_up]": @"☝",
        @"[下面]": @"👇",
        @"[左边]": @"👈",
        @"[右边]": @"👉",
        @"[raised_hands]": @"🙌",
        @"[pray]": @"🙏",
        @"[上面]": @"👆",
        @"[clap]": @"👏",
        @"[强壮]": @"💪",
        @"[walking]": @"🚶",
        @"[runner]": @"🏃",
        @"[running]": @"🏃",
        @"[couple]": @"👫",
        @"[family]": @"👪",
        @"[two_men_holding_hands]": @"👬",
        @"[two_women_holding_hands]": @"👭",
        @"[dancer]": @"💃",
        @"[dancers]": @"👯",
        @"[ok_woman]": @"🙆",
        @"[no_good]": @"🙅",
        @"[information_desk_person]": @"💁",
        @"[raised_hand]": @"🙋",
        @"[bride_with_veil]": @"👰",
        @"[person_with_pouting_face]": @"🙎",
        @"[person_frowning]": @"🙍",
        @"[bow]": @"🙇",
        @"[couplekiss]": @"💏",
        @"[couple_with_heart]": @"💑",
        @"[massage]": @"💆",
        @"[haircut]": @"💇",
        @"[nail_care]": @"💅",
        @"[boy]": @"👦",
        @"[girl]": @"👧",
        @"[woman]": @"👩",
        @"[man]": @"👨",
        @"[baby]": @"👶",
        @"[older_woman]": @"👵",
        @"[older_man]": @"👴",
        @"[person_with_blond_hair]": @"👱",
        @"[man_with_gua_pi_mao]": @"👲",
        @"[man_with_turban]": @"👳",
        @"[construction_worker]": @"👷",
        @"[cop]": @"👮",
        @"[angel]": @"👼",
        @"[princess]": @"👸",
        @"[smiley_cat]": @"😺",
        @"[smile_cat]": @"😸",
        @"[heart_eyes_cat]": @"😻",
        @"[kissing_cat]": @"😽",
        @"[smirk_cat]": @"😼",
        @"[scream_cat]": @"🙀",
        @"[crying_cat_face]": @"😿",
        @"[joy_cat]": @"😹",
        @"[pouting_cat]": @"😾",
        @"[japanese_ogre]": @"👹",
        @"[japanese_goblin]": @"👺",
        @"[see_no_evil]": @"🙈",
        @"[hear_no_evil]": @"🙉",
        @"[speak_no_evil]": @"🙊",
        @"[guardsman]": @"💂",
        @"[skull]": @"💀",
        @"[feet]": @"👣",
        @"[lips]": @"👄",
        @"[红唇]": @"💋",
        @"[droplet]": @"💧",
        @"[ear]": @"👂",
        @"[eyes]": @"👀",
        @"[nose]": @"👃",
        @"[tongue]": @"👅",
        @"[love_letter]": @"💌",
        @"[bust_in_silhouette]": @"👤",
        @"[busts_in_silhouette]": @"👥",
        @"[speech_balloon]": @"💬",
        @"[thought_balloon]": @"💭",
        @"[sunny]": @"☀",
        @"[umbrella]": @"☔",
        @"[cloud]": @"☁",
        @"[snowflake]": @"❄",
        @"[snowman]": @"⛄",
        @"[zap]": @"⚡",
        @"[cyclone]": @"🌀",
        @"[foggy]": @"🌁",
        @"[ocean]": @"🌊",
        @"[猫]": @"🐱",
        @"[小狗]": @"🐶",
        @"[老鼠]": @"🐭",
        @"[仓鼠]": @"🐹",
        @"[兔子]": @"🐰",
        @"[狼]": @"🐺",
        @"[青蛙]": @"🐸",
        @"[老虎]": @"🐯",
        @"[树袋熊]": @"🐨",
        @"[熊]": @"🐻",
        @"[猪头]": @"🐷",
        @"[pig_nose]": @"🐽",
        @"[cow]": @"🐮",
        @"[boar]": @"🐗",
        @"[猴子]": @"🐵",
        @"[猴]": @"🐒",
        @"[马]": @"🐴",
        @"[racehorse]": @"🐎",
        @"[骆驼]": @"🐫",
        @"[sheep]": @"🐑",
        @"[elephant]": @"🐘",
        @"[panda_face]": @"🐼",
        @"[snake]": @"🐍",
        @"[bird]": @"🐦",
        @"[baby_chick]": @"🐤",
        @"[hatched_chick]": @"🐥",
        @"[hatching_chick]": @"🐣",
        @"[chicken]": @"🐔",
        @"[penguin]": @"🐧",
        @"[turtle]": @"🐢",
        @"[bug]": @"🐛",
        @"[honeybee]": @"🐝",
        @"[ant]": @"🐜",
        @"[beetle]": @"🐞",
        @"[snail]": @"🐌",
        @"[octopus]": @"🐙",
        @"[tropical_fish]": @"🐠",
        @"[fish]": @"🐟",
        @"[whale]": @"🐳",
        @"[whale2]": @"🐋",
        @"[dolphin]": @"🐬",
        @"[cow2]": @"🐄",
        @"[ram]": @"🐏",
        @"[rat]": @"🐀",
        @"[water_buffalo]": @"🐃",
        @"[tiger2]": @"🐅",
        @"[rabbit2]": @"🐇",
        @"[dragon]": @"🐉",
        @"[goat]": @"🐐",
        @"[rooster]": @"🐓",
        @"[dog2]": @"🐕",
        @"[pig2]": @"🐖",
        @"[mouse2]": @"🐁",
        @"[ox]": @"🐂",
        @"[dragon_face]": @"🐲",
        @"[blowfish]": @"🐡",
        @"[crocodile]": @"🐊",
        @"[dromedary_camel]": @"🐪",
        @"[leopard]": @"🐆",
        @"[cat2]": @"🐈",
        @"[poodle]": @"🐩",
        @"[paw_prints]": @"🐾",
        @"[bouquet]": @"💐",
        @"[cherry_blossom]": @"🌸",
        @"[tulip]": @"🌷",
        @"[four_leaf_clover]": @"🍀",
        @"[rose]": @"🌹",
        @"[sunflower]": @"🌻",
        @"[hibiscus]": @"🌺",
        @"[maple_leaf]": @"🍁",
        @"[leaves]": @"🍃",
        @"[fallen_leaf]": @"🍂",
        @"[herb]": @"🌿",
        @"[mushroom]": @"🍄",
        @"[cactus]": @"🌵",
        @"[palm_tree]": @"🌴",
        @"[evergreen_tree]": @"🌲",
        @"[deciduous_tree]": @"🌳",
        @"[chestnut]": @"🌰",
        @"[seedling]": @"🌱",
        @"[blossum]": @"🌼",
        @"[ear_of_rice]": @"🌾",
        @"[shell]": @"🐚",
        @"[globe_with_meridians]": @"🌐",
        @"[sun_with_face]": @"🌞",
        @"[full_moon_with_face]": @"🌝",
        @"[new_moon_with_face]": @"🌚",
        @"[new_moon]": @"🌑",
        @"[waxing_crescent_moon]": @"🌒",
        @"[first_quarter_moon]": @"🌓",
        @"[waxing_gibbous_moon]": @"🌔",
        @"[full_moon]": @"🌕",
        @"[waning_gibbous_moon]": @"🌖",
        @"[last_quarter_moon]": @"🌗",
        @"[waning_crescent_moon]": @"🌘",
        @"[last_quarter_moon_with_face]": @"🌜",
        @"[first_quarter_moon_with_face]": @"🌛",
        @"[moon]": @"🌙",
        @"[earth_africa]": @"🌍",
        @"[earth_americas]": @"🌎",
        @"[earth_asia]": @"🌏",
        @"[volcano]": @"🌋",
        @"[milky_way]": @"🌌",
        @"[partly_sunny]": @"⛅",
        @"[bamboo]": @"🎍",
        @"[gift_heart]": @"💝",
        @"[dolls]": @"🎎",
        @"[school_satchel]": @"🎒",
        @"[mortar_board]": @"🎓",
        @"[flags]": @"🎏",
        @"[fireworks]": @"🎆",
        @"[sparkler]": @"🎇",
        @"[wind_chime]": @"🎐",
        @"[rice_scene]": @"🎑",
        @"[jack_o_lantern]": @"🎃",
        @"[ghost]": @"👻",
        @"[santa]": @"🎅",
        @"[8ball]": @"🎱",
        @"[alarm_clock]": @"⏰",
        @"[苹果]": @"🍎",
        @"[art]": @"🎨",
        @"[baby_bottle]": @"🍼",
        @"[balloon]": @"🎈",
        @"[banana]": @"🍌",
        @"[bar_chart]": @"📊",
        @"[baseball]": @"⚾",
        @"[basketball]": @"🏀",
        @"[bath]": @"🛀",
        @"[bathtub]": @"🛁",
        @"[battery]": @"🔋",
        @"[beer]": @"🍺",
        @"[beers]": @"🍻",
        @"[bell]": @"🔔",
        @"[bento]": @"🍱",
        @"[bicyclist]": @"🚴",
        @"[bikini]": @"👙",
        @"[birthday]": @"🎂",
        @"[black_joker]": @"🃏",
        @"[black_nib]": @"✒",
        @"[blue_book]": @"📘",
        @"[bomb]": @"💣",
        @"[bookmark]": @"🔖",
        @"[bookmark_tabs]": @"📑",
        @"[books]": @"📚",
        @"[boot]": @"👢",
        @"[bowling]": @"🎳",
        @"[bread]": @"🍞",
        @"[briefcase]": @"💼",
        @"[bulb]": @"💡",
        @"[cake]": @"🍰",
        @"[calendar]": @"📆",
        @"[calling]": @"📲",
        @"[camera]": @"📷",
        @"[candy]": @"🍬",
        @"[card_index]": @"📇",
        @"[cd]": @"💿",
        @"[chart_with_downwards_trend]": @"📉",
        @"[chart_with_upwards_trend]": @"📈",
        @"[cherries]": @"🍒",
        @"[chocolate_bar]": @"🍫",
        @"[christmas_tree]": @"🎄",
        @"[clapper]": @"🎬",
        @"[clipboard]": @"📋",
        @"[closed_book]": @"📕",
        @"[closed_lock_with_key]": @"🔐",
        @"[closed_umbrella]": @"🌂",
        @"[clubs]": @"♣",
        @"[cocktail]": @"🍸",
        @"[coffee]": @"☕",
        @"[computer]": @"💻",
        @"[confetti_ball]": @"🎊",
        @"[cookie]": @"🍪",
        @"[corn]": @"🌽",
        @"[credit_card]": @"💳",
        @"[crown]": @"👑",
        @"[crystal_ball]": @"🔮",
        @"[curry]": @"🍛",
        @"[custard]": @"🍮",
        @"[dango]": @"🍡",
        @"[dart]": @"🎯",
        @"[date]": @"📅",
        @"[diamonds]": @"♦",
        @"[dollar]": @"💵",
        @"[door]": @"🚪",
        @"[doughnut]": @"🍩",
        @"[dress]": @"👗",
        @"[dvd]": @"📀",
        @"[e-mail]": @"📧",
        @"[egg]": @"🍳",
        @"[eggplant]": @"🍆",
        @"[electric_plug]": @"🔌",
        @"[email]": @"✉",
        @"[envelope]": @"✉",
        @"[euro]": @"💶",
        @"[eyeglasses]": @"👓",
        @"[fax]": @"📠",
        @"[file_folder]": @"📁",
        @"[fish_cake]": @"🍥",
        @"[fishing_pole_and_fish]": @"🎣",
        @"[flashlight]": @"🔦",
        @"[软盘]": @"💾",
        @"[flower_playing_cards]": @"🎴",
        @"[football]": @"🏈",
        @"[fork_and_knife]": @"🍴",
        @"[fried_shrimp]": @"🍤",
        @"[fries]": @"🍟",
        @"[game_die]": @"🎲",
        @"[gem]": @"💎",
        @"[gift]": @"🎁",
        @"[golf]": @"⛳",
        @"[grapes]": @"🍇",
        @"[green_apple]": @"🍏",
        @"[green_book]": @"📗",
        @"[guitar]": @"🎸",
        @"[gun]": @"🔫",
        @"[hamburger]": @"🍔",
        @"[hammer]": @"🔨",
        @"[handbag]": @"👜",
        @"[headphones]": @"🎧",
        @"[hearts]": @"♥",
        @"[high_brightness]": @"🔆",
        @"[high_heel]": @"👠",
        @"[hocho]": @"🔪",
        @"[honey_pot]": @"🍯",
        @"[horse_racing]": @"🏇",
        @"[hourglass]": @"⌛",
        @"[hourglass_flowing_sand]": @"⏳",
        @"[ice_cream]": @"🍨",
        @"[icecream]": @"🍦",
        @"[inbox_tray]": @"📥",
        @"[incoming_envelope]": @"📨",
        @"[iphone]": @"📱",
        @"[jeans]": @"👖",
        @"[key]": @"🔑",
        @"[kimono]": @"👘",
        @"[ledger]": @"📒",
        @"[lemon]": @"🍋",
        @"[lipstick]": @"💄",
        @"[lock]": @"🔒",
        @"[lock_with_ink_pen]": @"🔏",
        @"[lollipop]": @"🍭",
        @"[loop]": @"➿",
        @"[loudspeaker]": @"📢",
        @"[low_brightness]": @"🔅",
        @"[mag]": @"🔍",
        @"[mag_right]": @"🔎",
        @"[mahjong]": @"🀄",
        @"[mailbox]": @"📫",
        @"[mailbox_closed]": @"📪",
        @"[mailbox_with_mail]": @"📬",
        @"[mailbox_with_no_mail]": @"📭",
        @"[mans_shoe]": @"👞",
        @"[shoe]": @"👞",
        @"[meat_on_bone]": @"🍖",
        @"[mega]": @"📣",
        @"[melon]": @"🍈",
        @"[memo]": @"📝",
        @"[pencil]": @"📝",
        @"[microphone]": @"🎤",
        @"[microscope]": @"🔬",
        @"[minidisc]": @"💽",
        @"[money_with_wings]": @"💸",
        @"[moneybag]": @"💰",
        @"[mountain_bicyclist]": @"🚵",
        @"[movie_camera]": @"🎥",
        @"[musical_keyboard]": @"🎹",
        @"[musical_score]": @"🎼",
        @"[mute]": @"🔇",
        @"[name_badge]": @"📛",
        @"[necktie]": @"👔",
        @"[newspaper]": @"📰",
        @"[no_bell]": @"🔕",
        @"[notebook]": @"📓",
        @"[notebook_with_decorative_cover]": @"📔",
        @"[nut_and_bolt]": @"🔩",
        @"[oden]": @"🍢",
        @"[open_file_folder]": @"📂",
        @"[orange_book]": @"📙",
        @"[outbox_tray]": @"📤",
        @"[page_facing_up]": @"📄",
        @"[page_with_curl]": @"📃",
        @"[pager]": @"📟",
        @"[paperclip]": @"📎",
        @"[peach]": @"🍑",
        @"[pear]": @"🍐",
        @"[pencil2]": @"✏",
        @"[phone]": @"☎",
        @"[telephone]": @"☎",
        @"[pill]": @"💊",
        @"[pineapple]": @"🍍",
        @"[pizza]": @"🍕",
        @"[postal_horn]": @"📯",
        @"[postbox]": @"📮",
        @"[pouch]": @"👝",
        @"[poultry_leg]": @"🍗",
        @"[pound]": @"💷",
        @"[purse]": @"👛",
        @"[pushpin]": @"📌",
        @"[radio]": @"📻",
        @"[ramen]": @"🍜",
        @"[ribbon]": @"🎀",
        @"[rice]": @"🍚",
        @"[rice_ball]": @"🍙",
        @"[rice_cracker]": @"🍘",
        @"[ring]": @"💍",
        @"[rugby_football]": @"🏉",
        @"[running_shirt_with_sash]": @"🎽",
        @"[sake]": @"🍶",
        @"[sandal]": @"👡",
        @"[satellite]": @"📡",
        @"[saxophone]": @"🎷",
        @"[scissors]": @"✂",
        @"[scroll]": @"📜",
        @"[seat]": @"💺",
        @"[shaved_ice]": @"🍧",
        @"[shirt]": @"👕",
        @"[tshirt]": @"👕",
        @"[shower]": @"🚿",
        @"[ski]": @"🎿",
        @"[smoking]": @"🚬",
        @"[snowboarder]": @"🏂",
        @"[soccer]": @"⚽",
        @"[sound]": @"🔉",
        @"[space_invader]": @"👾",
        @"[spades]": @"♠",
        @"[spaghetti]": @"🍝",
        @"[speaker]": @"🔊",
        @"[stew]": @"🍲",
        @"[straight_ruler]": @"📏",
        @"[strawberry]": @"🍓",
        @"[surfer]": @"🏄",
        @"[sushi]": @"🍣",
        @"[sweet_potato]": @"🍠",
        @"[swimmer]": @"🏊",
        @"[syringe]": @"💉",
        @"[tada]": @"🎉",
        @"[tanabata_tree]": @"🎋",
        @"[tangerine]": @"🍊",
        @"[tea]": @"🍵",
        @"[telephone_receiver]": @"📞",
        @"[telescope]": @"🔭",
        @"[tennis]": @"🎾",
        @"[toilet]": @"🚽",
        @"[tomato]": @"🍅",
        @"[tophat]": @"🎩",
        @"[triangular_ruler]": @"📐",
        @"[trophy]": @"🏆",
        @"[tropical_drink]": @"🍹",
        @"[trumpet]": @"🎺",
        @"[tv]": @"📺",
        @"[unlock]": @"🔓",
        @"[vhs]": @"📼",
        @"[video_camera]": @"📹",
        @"[video_game]": @"🎮",
        @"[violin]": @"🎻",
        @"[watch]": @"⌚",
        @"[watermelon]": @"🍉",
        @"[wine_glass]": @"🍷",
        @"[womans_clothes]": @"👚",
        @"[womans_hat]": @"👒",
        @"[wrench]": @"🔧",
        @"[yen]": @"💴",
        @"[aerial_tramway]": @"🚡",
        @"[airplane]": @"✈",
        @"[ambulance]": @"🚑",
        @"[anchor]": @"⚓",
        @"[articulated_lorry]": @"🚛",
        @"[atm]": @"🏧",
        @"[bank]": @"🏦",
        @"[barber]": @"💈",
        @"[beginner]": @"🔰",
        @"[bike]": @"🚲",
        @"[blue_car]": @"🚙",
        @"[boat]": @"⛵",
        @"[sailboat]": @"⛵",
        @"[bridge_at_night]": @"🌉",
        @"[bullettrain_front]": @"🚅",
        @"[bullettrain_side]": @"🚄",
        @"[bus]": @"🚌",
        @"[busstop]": @"🚏",
        @"[car]": @"🚗",
        @"[red_car]": @"🚗",
        @"[carousel_horse]": @"🎠",
        @"[checkered_flag]": @"🏁",
        @"[church]": @"⛪",
        @"[circus_tent]": @"🎪",
        @"[city_sunrise]": @"🌇",
        @"[city_sunset]": @"🌆",
        @"[construction]": @"🚧",
        @"[convenience_store]": @"🏪",
        @"[crossed_flags]": @"🎌",
        @"[department_store]": @"🏬",
        @"[european_castle]": @"🏰",
        @"[european_post_office]": @"🏤",
        @"[factory]": @"🏭",
        @"[ferris_wheel]": @"🎡",
        @"[fire_engine]": @"🚒",
        @"[fountain]": @"⛲",
        @"[fuelpump]": @"⛽",
        @"[helicopter]": @"🚁",
        @"[hospital]": @"🏥",
        @"[hotel]": @"🏨",
        @"[hotsprings]": @"♨",
        @"[house]": @"🏠",
        @"[house_with_garden]": @"🏡",
        @"[japan]": @"🗾",
        @"[japanese_castle]": @"🏯",
        @"[light_rail]": @"🚈",
        @"[love_hotel]": @"🏩",
        @"[minibus]": @"🚐",
        @"[monorail]": @"🚝",
        @"[mount_fuji]": @"🗻",
        @"[mountain_cableway]": @"🚠",
        @"[mountain_railway]": @"🚞",
        @"[moyai]": @"🗿",
        @"[office]": @"🏢",
        @"[oncoming_automobile]": @"🚘",
        @"[oncoming_bus]": @"🚍",
        @"[oncoming_police_car]": @"🚔",
        @"[oncoming_taxi]": @"🚖",
        @"[performing_arts]": @"🎭",
        @"[police_car]": @"🚓",
        @"[post_office]": @"🏣",
        @"[railway_car]": @"🚃",
        @"[train]": @"🚃",
        @"[rainbow]": @"🌈",
        @"[rocket]": @"🚀",
        @"[roller_coaster]": @"🎢",
        @"[rotating_light]": @"🚨",
        @"[round_pushpin]": @"📍",
        @"[rowboat]": @"🚣",
        @"[school]": @"🏫",
        @"[ship]": @"🚢",
        @"[slot_machine]": @"🎰",
        @"[speedboat]": @"🚤",
        @"[stars]": @"🌃",
        @"[station]": @"🚉",
        @"[statue_of_liberty]": @"🗽",
        @"[steam_locomotive]": @"🚂",
        @"[sunrise]": @"🌅",
        @"[sunrise_over_mountains]": @"🌄",
        @"[suspension_railway]": @"🚟",
        @"[taxi]": @"🚕",
        @"[tent]": @"⛺",
        @"[ticket]": @"🎫",
        @"[tokyo_tower]": @"🗼",
        @"[tractor]": @"🚜",
        @"[traffic_light]": @"🚥",
        @"[train2]": @"🚆",
        @"[tram]": @"🚊",
        @"[triangular_flag_on_post]": @"🚩",
        @"[trolleybus]": @"🚎",
        @"[truck]": @"🚚",
        @"[vertical_traffic_light]": @"🚦",
        @"[warning]": @"⚠",
        @"[wedding]": @"💒",
        @"[jp]": @"🇯🇵",
        @"[kr]": @"🇰🇷",
        @"[cn]": @"🇨🇳",
        @"[us]": @"🇺🇸",
        @"[fr]": @"🇫🇷",
        @"[es]": @"🇪🇸",
        @"[it]": @"🇮🇹",
        @"[ru]": @"🇷🇺",
        @"[gb]": @"🇬🇧",
        @"[uk]": @"🇬🇧",
        @"[de]": @"🇩🇪",
        @"[100]": @"💯",
        @"[1234]": @"🔢",
        @"[a]": @"🅰",
        @"[ab]": @"🆎",
        @"[abc]": @"🔤",
        @"[abcd]": @"🔡",
        @"[accept]": @"🉑",
        @"[aquarius]": @"♒",
        @"[aries]": @"♈",
        @"[arrow_backward]": @"◀",
        @"[arrow_double_down]": @"⏬",
        @"[arrow_double_up]": @"⏫",
        @"[arrow_down]": @"⬇",
        @"[arrow_down_small]": @"🔽",
        @"[arrow_forward]": @"▶",
        @"[arrow_heading_down]": @"⤵",
        @"[arrow_heading_up]": @"⤴",
        @"[arrow_left]": @"⬅",
        @"[arrow_lower_left]": @"↙",
        @"[arrow_lower_right]": @"↘",
        @"[arrow_right]": @"➡",
        @"[arrow_right_hook]": @"↪",
        @"[arrow_up]": @"⬆",
        @"[arrow_up_down]": @"↕",
        @"[arrow_up_small]": @"🔼",
        @"[arrow_upper_left]": @"↖",
        @"[arrow_upper_right]": @"↗",
        @"[arrows_clockwise]": @"🔃",
        @"[arrows_counterclockwise]": @"🔄",
        @"[b]": @"🅱",
        @"[baby_symbol]": @"🚼",
        @"[baggage_claim]": @"🛄",
        @"[ballot_box_with_check]": @"☑",
        @"[bangbang]": @"‼",
        @"[black_circle]": @"⚫",
        @"[black_square_button]": @"🔲",
        @"[cancer]": @"♋",
        @"[capital_abcd]": @"🔠",
        @"[capricorn]": @"♑",
        @"[chart]": @"💹",
        @"[children_crossing]": @"🚸",
        @"[cinema]": @"🎦",
        @"[cl]": @"🆑",
        @"[clock1]": @"🕐",
        @"[clock10]": @"🕙",
        @"[clock1030]": @"🕥",
        @"[clock11]": @"🕚",
        @"[clock1130]": @"🕦",
        @"[clock12]": @"🕛",
        @"[clock1230]": @"🕧",
        @"[clock130]": @"🕜",
        @"[clock2]": @"🕑",
        @"[clock230]": @"🕝",
        @"[clock3]": @"🕒",
        @"[clock330]": @"🕞",
        @"[clock4]": @"🕓",
        @"[clock430]": @"🕟",
        @"[clock5]": @"🕔",
        @"[clock530]": @"🕠",
        @"[clock6]": @"🕕",
        @"[clock630]": @"🕡",
        @"[clock7]": @"🕖",
        @"[clock730]": @"🕢",
        @"[clock8]": @"🕗",
        @"[clock830]": @"🕣",
        @"[clock9]": @"🕘",
        @"[clock930]": @"🕤",
        @"[congratulations]": @"㊗",
        @"[cool]": @"🆒",
        @"[copyright]": @"©",
        @"[curly_loop]": @"➰",
        @"[currency_exchange]": @"💱",
        @"[customs]": @"🛃",
        @"[diamond_shape_with_a_dot_inside]": @"💠",
        @"[do_not_litter]": @"🚯",
        @"[eight]": @"8⃣",
        @"[eight_pointed_black_star]": @"✴",
        @"[eight_spoked_asterisk]": @"✳",
        @"[end]": @"🔚",
        @"[fast_forward]": @"⏩",
        @"[five]": @"5⃣",
        @"[four]": @"4⃣",
        @"[free]": @"🆓",
        @"[gemini]": @"♊",
        @"[hash]": @"#⃣",
        @"[heart_decoration]": @"💟",
        @"[heavy_check_mark]": @"✔",
        @"[heavy_division_sign]": @"➗",
        @"[heavy_dollar_sign]": @"💲",
        @"[heavy_minus_sign]": @"➖",
        @"[heavy_multiplication_x]": @"✖",
        @"[heavy_plus_sign]": @"➕",
        @"[id]": @"🆔",
        @"[ideograph_advantage]": @"🉐",
        @"[information_source]": @"ℹ",
        @"[interrobang]": @"⁉",
        @"[keycap_ten]": @"🔟",
        @"[koko]": @"🈁",
        @"[large_blue_circle]": @"🔵",
        @"[large_blue_diamond]": @"🔷",
        @"[large_orange_diamond]": @"🔶",
        @"[left_luggage]": @"🛅",
        @"[left_right_arrow]": @"↔",
        @"[leftwards_arrow_with_hook]": @"↩",
        @"[leo]": @"♌",
        @"[libra]": @"♎",
        @"[link]": @"🔗",
        @"[m]": @"Ⓜ",
        @"[mens]": @"🚹",
        @"[metro]": @"🚇",
        @"[mobile_phone_off]": @"📴",
        @"[negative_squared_cross_mark]": @"❎",
        @"[new]": @"🆕",
        @"[ng]": @"🆖",
        @"[nine]": @"9⃣",
        @"[no_bicycles]": @"🚳",
        @"[no_entry]": @"⛔",
        @"[no_entry_sign]": @"🚫",
        @"[no_mobile_phones]": @"📵",
        @"[no_pedestrians]": @"🚷",
        @"[no_smoking]": @"🚭",
        @"[non-potable_water]": @"🚱",
        @"[o]": @"⭕",
        @"[o2]": @"🅾",
        @"[ok]": @"🆗",
        @"[on]": @"🔛",
        @"[one]": @"1⃣",
        @"[ophiuchus]": @"⛎",
        @"[parking]": @"🅿",
        @"[part_alternation_mark]": @"〽",
        @"[passport_control]": @"🛂",
        @"[pisces]": @"♓",
        @"[potable_water]": @"🚰",
        @"[put_litter_in_its_place]": @"🚮",
        @"[radio_button]": @"🔘",
        @"[recycle]": @"♻",
        @"[red_circle]": @"🔴",
        @"[registered]": @"®",
        @"[repeat]": @"🔁",
        @"[repeat_one]": @"🔂",
        @"[restroom]": @"🚻",
        @"[rewind]": @"⏪",
        @"[sa]": @"🈂",
        @"[sagittarius]": @"♐",
        @"[scorpius]": @"♏",
        @"[secret]": @"㊙",
        @"[seven]": @"7⃣",
        @"[signal_strength]": @"📶",
        @"[six]": @"6⃣",
        @"[six_pointed_star]": @"🔯",
        @"[small_blue_diamond]": @"🔹",
        @"[small_orange_diamond]": @"🔸",
        @"[small_red_triangle]": @"🔺",
        @"[small_red_triangle_down]": @"🔻",
        @"[soon]": @"🔜",
        @"[sos]": @"🆘",
        @"[symbols]": @"🔣",
        @"[taurus]": @"♉",
        @"[three]": @"3⃣",
        @"[tm]": @"™",
        @"[top]": @"🔝",
        @"[trident]": @"🔱",
        @"[twisted_rightwards_arrows]": @"🔀",
        @"[two]": @"2⃣",
        @"[u5272]": @"🈹",
        @"[u5408]": @"🈴",
        @"[u55b6]": @"🈺",
        @"[u6307]": @"🈯",
        @"[u6708]": @"🈷",
        @"[u6709]": @"🈶",
        @"[u6e80]": @"🈵",
        @"[u7121]": @"🈚",
        @"[u7533]": @"🈸",
        @"[u7981]": @"🈲",
        @"[u7a7a]": @"🈳",
        @"[underage]": @"🔞",
        @"[up]": @"🆙",
        @"[vibration_mode]": @"📳",
        @"[virgo]": @"♍",
        @"[vs]": @"🆚",
        @"[wavy_dash]": @"〰",
        @"[wc]": @"🚾",
        @"[wheelchair]": @"♿",
        @"[white_check_mark]": @"✅",
        @"[white_circle]": @"⚪",
        @"[white_flower]": @"💮",
        @"[white_square_button]": @"🔳",
        @"[womens]": @"🚺",
        @"[x]": @"❌",
        @"[zero]": @"0⃣",
							 };
	
	__block NSString *text = self;
	
	[emojis enumerateKeysAndObjectsUsingBlock:^(id key, id obj, BOOL *stop) {
		text = [text stringByReplacingOccurrencesOfString:key withString:obj];
	}];
	
	return text;
}

- (NSString *)replaceEmojiUnicodeWithText {
	
	NSDictionary *emojis = @{
				@"😄": @"[哈哈]",
				@"😆": @"[laughing]",
				@"😊": @"[blush]",
				@"😃": @"[微笑]",
				@"☺": @"[relaxed]",
				@"😏": @"[假笑]",
				@"😍": @"[heart_eyes]",
				@"😘": @"[kissing_heart]",
				@"😚": @"[亲吻]",
				@"😳": @"[脸红]",
				@"😥": @"[relieved]",
				@"😌": @"[满足]",
				@"😁": @"[grin]",
				@"😉": @"[wink]",
				@"😜": @"[鬼脸]",
				@"😝": @"[闭眼]",
				@"😀": @"[grinning]",
				@"😗": @"[kissing]",
				@"😙": @"[kissing_smiling_eyes]",
				@"😛": @"[stuck_out_tongue]",
				@"😴": @"[sleeping]",
				@"😟": @"[worried]",
				@"😦": @"[frowning]",
				@"😧": @"[anguished]",
				@"😮": @"[open_mouth]",
				@"😬": @"[grimacing]",
				@"😕": @"[confused]",
				@"😯": @"[hushed]",
				@"😑": @"[expressionless]",
				@"😒": @"[鄙视]",
				@"😅": @"[sweat_smile]",
				@"😓": @"[冷汗]",
				@"😩": @"[weary]",
				@"😔": @"[pensive]",
				@"😞": @"[失望]",
				@"😖": @"[confounded]",
				@"😨": @"[害怕]",
				@"😰": @"[糟糕]",
				@"😣": @"[persevere]",
				@"😢": @"[悲伤]",
				@"😭": @"[大哭]",
				@"😂": @"[joy]",
				@"😲": @"[astonished]",
				@"😱": @"[恐怖]",
				@"😫": @"[tired_face]",
				@"😠": @"[不高兴]",
				@"😡": @"[愤怒]",
				@"😤": @"[triumph]",
				@"😪": @"[sleepy]",
				@"😋": @"[yum]",
				@"😷": @"[生病]",
				@"😎": @"[sunglasses]",
				@"😵": @"[dizzy_face]",
				@"👿": @"[恶魔]",
				@"😈": @"[smiling_imp]",
				@"😐": @"[neutral_face]",
				@"😶": @"[no_mouth]",
				@"😇": @"[innocent]",
				@"👽": @"[外星人]",
				@"💛": @"[黄心]",
				@"💙": @"[blue_heart]",
				@"💜": @"[purple_heart]",
				@"❤": @"[心]",
				@"💚": @"[green_heart]",
				@"💔": @"[心碎]",
				@"💓": @"[heartbeat]",
				@"💗": @"[heartpulse]",
				@"💕": @"[two_hearts]",
				@"💞": @"[revolving_hearts]",
				@"💘": @"[cupid]",
				@"💖": @"[sparkling_heart]",
				@"✨": @"[星星]",
				@"⭐": @"[star]",
				@"🌟": @"[star2]",
				@"💫": @"[dizzy]",
				@"💥": @"[boom]",
				@"💢": @"[anger]",
				@"❗": @"[exclamation]",
				@"❓": @"[question]",
				@"❕": @"[grey_exclamation]",
				@"❔": @"[grey_question]",
				@"💤": @"[zzz]",
				@"💨": @"[dash]",
				@"💦": @"[sweat_drops]",
				@"🎶": @"[notes]",
				@"🎵": @"[音符]",
				@"🔥": @"[火焰]",
				@"💩": @"[hankey]",
				@"👍": @"[点赞]",
				@"👎": @"[thumbsdown]",
				@"👌": @"[ok_hand]",
				@"👊": @"[punch]",
				@"✊": @"[fist]",
				@"✌": @"[耶]",
				@"👋": @"[挥手]",
				@"✋": @"[hand]",
				@"👐": @"[open_hands]",
				@"☝": @"[point_up]",
				@"👇": @"[下面]",
				@"👈": @"[左边]",
				@"👉": @"[右边]",
				@"🙌": @"[raised_hands]",
				@"🙏": @"[pray]",
				@"👆": @"[上面]",
				@"👏": @"[clap]",
				@"💪": @"[强壮]",
				@"🚶": @"[walking]",
				@"🏃": @"[runner]",
				@"👫": @"[couple]",
				@"👪": @"[family]",
				@"👬": @"[two_men_holding_hands]",
				@"👭": @"[two_women_holding_hands]",
				@"💃": @"[dancer]",
				@"👯": @"[dancers]",
				@"🙆": @"[ok_woman]",
				@"🙅": @"[no_good]",
				@"💁": @"[information_desk_person]",
				@"🙋": @"[raised_hand]",
				@"👰": @"[bride_with_veil]",
				@"🙎": @"[person_with_pouting_face]",
				@"🙍": @"[person_frowning]",
				@"🙇": @"[bow]",
				@"💏": @"[couplekiss]",
				@"💑": @"[couple_with_heart]",
				@"💆": @"[massage]",
				@"💇": @"[haircut]",
				@"💅": @"[nail_care]",
				@"👦": @"[boy]",
				@"👧": @"[girl]",
				@"👩": @"[woman]",
				@"👨": @"[man]",
				@"👶": @"[baby]",
				@"👵": @"[older_woman]",
				@"👴": @"[older_man]",
				@"👱": @"[person_with_blond_hair]",
				@"👲": @"[man_with_gua_pi_mao]",
				@"👳": @"[man_with_turban]",
				@"👷": @"[construction_worker]",
				@"👮": @"[cop]",
				@"👼": @"[angel]",
				@"👸": @"[princess]",
				@"😺": @"[smiley_cat]",
				@"😸": @"[smile_cat]",
				@"😻": @"[heart_eyes_cat]",
				@"😽": @"[kissing_cat]",
				@"😼": @"[smirk_cat]",
				@"🙀": @"[scream_cat]",
				@"😿": @"[crying_cat_face]",
				@"😹": @"[joy_cat]",
				@"😾": @"[pouting_cat]",
				@"👹": @"[japanese_ogre]",
				@"👺": @"[japanese_goblin]",
				@"🙈": @"[see_no_evil]",
				@"🙉": @"[hear_no_evil]",
				@"🙊": @"[speak_no_evil]",
				@"💂": @"[guardsman]",
				@"💀": @"[skull]",
				@"👣": @"[feet]",
				@"👄": @"[lips]",
				@"💋": @"[红唇]",
				@"💧": @"[droplet]",
				@"👂": @"[ear]",
				@"👀": @"[eyes]",
				@"👃": @"[nose]",
				@"👅": @"[tongue]",
				@"💌": @"[love_letter]",
				@"👤": @"[bust_in_silhouette]",
				@"👥": @"[busts_in_silhouette]",
				@"💬": @"[speech_balloon]",
				@"💭": @"[thought_balloon]",
				@"☀": @"[太阳]",
				@"☔": @"[雨伞]",
				@"☁": @"[多云]",
				@"❄": @"[snowflake]",
				@"⛄": @"[雪人]",
				@"⚡": @"[zap]",
				@"🌀": @"[cyclone]",
				@"🌁": @"[foggy]",
				@"🌊": @"[波浪]",
				@"🐱": @"[猫]",
				@"🐶": @"[小狗]",
				@"🐭": @"[老鼠]",
				@"🐹": @"[仓鼠]",
				@"🐰": @"[兔子]",
				@"🐺": @"[狼]",
				@"🐸": @"[青蛙]",
				@"🐯": @"[老虎]",
				@"🐨": @"[树袋熊]",
				@"🐻": @"[熊]",
				@"🐷": @"[猪头]",
				@"🐽": @"[pig_nose]",
				@"🐮": @"[cow]",
				@"🐗": @"[boar]",
				@"🐵": @"[猴子]",
				@"🐒": @"[猴]",
				@"🐴": @"[马]",
				@"🐎": @"[racehorse]",
				@"🐫": @"[camel]",
				@"🐑": @"[sheep]",
				@"🐘": @"[elephant]",
				@"🐼": @"[panda_face]",
				@"🐍": @"[snake]",
				@"🐦": @"[bird]",
				@"🐤": @"[baby_chick]",
				@"🐥": @"[hatched_chick]",
				@"🐣": @"[hatching_chick]",
				@"🐔": @"[chicken]",
				@"🐧": @"[penguin]",
				@"🐢": @"[turtle]",
				@"🐛": @"[bug]",
				@"🐝": @"[honeybee]",
				@"🐜": @"[ant]",
				@"🐞": @"[beetle]",
				@"🐌": @"[snail]",
				@"🐙": @"[octopus]",
				@"🐠": @"[tropical_fish]",
				@"🐟": @"[fish]",
				@"🐳": @"[whale]",
				@"🐋": @"[whale2]",
				@"🐬": @"[dolphin]",
				@"🐄": @"[cow2]",
				@"🐏": @"[ram]",
				@"🐀": @"[rat]",
				@"🐃": @"[water_buffalo]",
				@"🐅": @"[tiger2]",
				@"🐇": @"[rabbit2]",
				@"🐉": @"[dragon]",
				@"🐐": @"[goat]",
				@"🐓": @"[rooster]",
				@"🐕": @"[dog2]",
				@"🐖": @"[pig2]",
				@"🐁": @"[mouse2]",
				@"🐂": @"[ox]",
				@"🐲": @"[dragon_face]",
				@"🐡": @"[blowfish]",
				@"🐊": @"[crocodile]",
				@"🐪": @"[dromedary_camel]",
				@"🐆": @"[leopard]",
				@"🐈": @"[cat2]",
				@"🐩": @"[poodle]",
				@"🐾": @"[paw_prints]",
				@"💐": @"[bouquet]",
				@"🌸": @"[cherry_blossom]",
				@"🌷": @"[tulip]",
				@"🍀": @"[four_leaf_clover]",
				@"🌹": @"[rose]",
				@"🌻": @"[sunflower]",
				@"🌺": @"[hibiscus]",
				@"🍁": @"[maple_leaf]",
				@"🍃": @"[leaves]",
				@"🍂": @"[fallen_leaf]",
				@"🌿": @"[herb]",
				@"🍄": @"[mushroom]",
				@"🌵": @"[cactus]",
				@"🌴": @"[palm_tree]",
				@"🌲": @"[evergreen_tree]",
				@"🌳": @"[deciduous_tree]",
				@"🌰": @"[chestnut]",
				@"🌱": @"[seedling]",
				@"🌼": @"[blossum]",
				@"🌾": @"[ear_of_rice]",
				@"🐚": @"[shell]",
				@"🌐": @"[globe_with_meridians]",
				@"🌞": @"[sun_with_face]",
				@"🌝": @"[full_moon_with_face]",
				@"🌚": @"[new_moon_with_face]",
				@"🌑": @"[new_moon]",
				@"🌒": @"[waxing_crescent_moon]",
				@"🌓": @"[first_quarter_moon]",
				@"🌔": @"[waxing_gibbous_moon]",
				@"🌕": @"[full_moon]",
				@"🌖": @"[waning_gibbous_moon]",
				@"🌗": @"[last_quarter_moon]",
				@"🌘": @"[waning_crescent_moon]",
				@"🌜": @"[last_quarter_moon_with_face]",
				@"🌛": @"[first_quarter_moon_with_face]",
				@"🌙": @"[moon]",
				@"🌍": @"[earth_africa]",
				@"🌎": @"[earth_americas]",
				@"🌏": @"[earth_asia]",
				@"🌋": @"[volcano]",
				@"🌌": @"[milky_way]",
				@"⛅": @"[partly_sunny]",
				@"🎍": @"[bamboo]",
				@"💝": @"[gift_heart]",
				@"🎎": @"[dolls]",
				@"🎒": @"[school_satchel]",
				@"🎓": @"[mortar_board]",
				@"🎏": @"[flags]",
				@"🎆": @"[fireworks]",
				@"🎇": @"[sparkler]",
				@"🎐": @"[wind_chime]",
				@"🎑": @"[rice_scene]",
				@"🎃": @"[jack_o_lantern]",
				@"👻": @"[ghost]",
				@"🎅": @"[santa]",
				@"🎱": @"[8ball]",
				@"⏰": @"[alarm_clock]",
				@"🍎": @"[apple]",
				@"🎨": @"[art]",
				@"🍼": @"[baby_bottle]",
				@"🎈": @"[balloon]",
				@"🍌": @"[banana]",
				@"📊": @"[bar_chart]",
				@"⚾": @"[baseball]",
				@"🏀": @"[basketball]",
				@"🛀": @"[bath]",
				@"🛁": @"[bathtub]",
				@"🔋": @"[battery]",
				@"🍺": @"[beer]",
				@"🍻": @"[beers]",
				@"🔔": @"[bell]",
				@"🍱": @"[bento]",
				@"🚴": @"[bicyclist]",
				@"👙": @"[bikini]",
				@"🎂": @"[birthday]",
				@"🃏": @"[black_joker]",
				@"✒": @"[black_nib]",
				@"📘": @"[blue_book]",
				@"💣": @"[bomb]",
				@"🔖": @"[bookmark]",
				@"📑": @"[bookmark_tabs]",
				@"📚": @"[books]",
				@"👢": @"[boot]",
				@"🎳": @"[bowling]",
				@"🍞": @"[bread]",
				@"💼": @"[briefcase]",
				@"💡": @"[bulb]",
				@"🍰": @"[cake]",
				@"📆": @"[calendar]",
				@"📲": @"[calling]",
				@"📷": @"[camera]",
				@"🍬": @"[candy]",
				@"📇": @"[card_index]",
				@"💿": @"[cd]",
				@"📉": @"[chart_with_downwards_trend]",
				@"📈": @"[chart_with_upwards_trend]",
				@"🍒": @"[cherries]",
				@"🍫": @"[chocolate_bar]",
				@"🎄": @"[christmas_tree]",
				@"🎬": @"[clapper]",
				@"📋": @"[clipboard]",
				@"📕": @"[closed_book]",
				@"🔐": @"[closed_lock_with_key]",
				@"🌂": @"[closed_umbrella]",
				@"♣": @"[clubs]",
				@"🍸": @"[cocktail]",
				@"☕": @"[coffee]",
				@"💻": @"[computer]",
				@"🎊": @"[confetti_ball]",
				@"🍪": @"[cookie]",
				@"🌽": @"[corn]",
				@"💳": @"[credit_card]",
				@"👑": @"[crown]",
				@"🔮": @"[crystal_ball]",
				@"🍛": @"[curry]",
				@"🍮": @"[custard]",
				@"🍡": @"[dango]",
				@"🎯": @"[dart]",
				@"📅": @"[date]",
				@"♦": @"[diamonds]",
				@"💵": @"[dollar]",
				@"🚪": @"[door]",
				@"🍩": @"[doughnut]",
				@"👗": @"[dress]",
				@"📀": @"[dvd]",
				@"📧": @"[e-mail]",
				@"🍳": @"[egg]",
				@"🍆": @"[eggplant]",
				@"🔌": @"[electric_plug]",
				@"✉": @"[email]",
				@"💶": @"[euro]",
				@"👓": @"[eyeglasses]",
				@"📠": @"[fax]",
				@"📁": @"[file_folder]",
				@"🍥": @"[fish_cake]",
				@"🎣": @"[fishing_pole_and_fish]",
				@"🔦": @"[flashlight]",
				@"💾": @"[floppy_disk]",
				@"🎴": @"[flower_playing_cards]",
				@"🏈": @"[football]",
				@"🍴": @"[fork_and_knife]",
				@"🍤": @"[fried_shrimp]",
				@"🍟": @"[fries]",
				@"🎲": @"[game_die]",
				@"💎": @"[gem]",
				@"🎁": @"[gift]",
				@"⛳": @"[golf]",
				@"🍇": @"[grapes]",
				@"🍏": @"[green_apple]",
				@"📗": @"[green_book]",
				@"🎸": @"[guitar]",
				@"🔫": @"[gun]",
				@"🍔": @"[hamburger]",
				@"🔨": @"[hammer]",
				@"👜": @"[handbag]",
				@"🎧": @"[headphones]",
				@"♥": @"[hearts]",
				@"🔆": @"[high_brightness]",
				@"👠": @"[high_heel]",
				@"🔪": @"[hocho]",
				@"🍯": @"[honey_pot]",
				@"🏇": @"[horse_racing]",
				@"⌛": @"[hourglass]",
				@"⏳": @"[hourglass_flowing_sand]",
				@"🍨": @"[ice_cream]",
				@"🍦": @"[icecream]",
				@"📥": @"[inbox_tray]",
				@"📨": @"[incoming_envelope]",
				@"📱": @"[iphone]",
				@"👖": @"[jeans]",
				@"🔑": @"[key]",
				@"👘": @"[kimono]",
				@"📒": @"[ledger]",
				@"🍋": @"[lemon]",
				@"💄": @"[lipstick]",
				@"🔒": @"[lock]",
				@"🔏": @"[lock_with_ink_pen]",
				@"🍭": @"[lollipop]",
				@"➿": @"[loop]",
				@"📢": @"[loudspeaker]",
				@"🔅": @"[low_brightness]",
				@"🔍": @"[mag]",
				@"🔎": @"[mag_right]",
				@"🀄": @"[mahjong]",
				@"📫": @"[mailbox]",
				@"📪": @"[mailbox_closed]",
				@"📬": @"[mailbox_with_mail]",
				@"📭": @"[mailbox_with_no_mail]",
				@"👞": @"[mans_shoe]",
				@"🍖": @"[meat_on_bone]",
				@"📣": @"[mega]",
				@"🍈": @"[melon]",
				@"📝": @"[memo]",
				@"🎤": @"[microphone]",
				@"🔬": @"[microscope]",
				@"💽": @"[minidisc]",
				@"💸": @"[money_with_wings]",
				@"💰": @"[moneybag]",
				@"🚵": @"[mountain_bicyclist]",
				@"🎥": @"[movie_camera]",
				@"🎹": @"[musical_keyboard]",
				@"🎼": @"[musical_score]",
				@"🔇": @"[mute]",
				@"📛": @"[name_badge]",
				@"👔": @"[necktie]",
				@"📰": @"[newspaper]",
				@"🔕": @"[no_bell]",
				@"📓": @"[notebook]",
				@"📔": @"[notebook_with_decorative_cover]",
				@"🔩": @"[nut_and_bolt]",
				@"🍢": @"[oden]",
				@"📂": @"[open_file_folder]",
				@"📙": @"[orange_book]",
				@"📤": @"[outbox_tray]",
				@"📄": @"[page_facing_up]",
				@"📃": @"[page_with_curl]",
				@"📟": @"[pager]",
				@"📎": @"[paperclip]",
				@"🍑": @"[peach]",
				@"🍐": @"[pear]",
				@"✏": @"[pencil2]",
				@"☎": @"[phone]",
				@"💊": @"[pill]",
				@"🍍": @"[pineapple]",
				@"🍕": @"[pizza]",
				@"📯": @"[postal_horn]",
				@"📮": @"[postbox]",
				@"👝": @"[pouch]",
				@"🍗": @"[poultry_leg]",
				@"💷": @"[pound]",
				@"👛": @"[purse]",
				@"📌": @"[pushpin]",
				@"📻": @"[radio]",
				@"🍜": @"[ramen]",
				@"🎀": @"[ribbon]",
				@"🍚": @"[rice]",
				@"🍙": @"[rice_ball]",
				@"🍘": @"[rice_cracker]",
				@"💍": @"[ring]",
				@"🏉": @"[rugby_football]",
				@"🎽": @"[running_shirt_with_sash]",
				@"🍶": @"[sake]",
				@"👡": @"[sandal]",
				@"📡": @"[satellite]",
				@"🎷": @"[saxophone]",
				@"✂": @"[scissors]",
				@"📜": @"[scroll]",
				@"💺": @"[seat]",
				@"🍧": @"[shaved_ice]",
				@"👕": @"[shirt]",
				@"🚿": @"[shower]",
				@"🎿": @"[ski]",
				@"🚬": @"[smoking]",
				@"🏂": @"[snowboarder]",
				@"⚽": @"[soccer]",
				@"🔉": @"[sound]",
				@"👾": @"[space_invader]",
				@"♠": @"[spades]",
				@"🍝": @"[spaghetti]",
				@"🔊": @"[speaker]",
				@"🍲": @"[stew]",
				@"📏": @"[straight_ruler]",
				@"🍓": @"[strawberry]",
				@"🏄": @"[surfer]",
				@"🍣": @"[sushi]",
				@"🍠": @"[sweet_potato]",
				@"🏊": @"[swimmer]",
				@"💉": @"[syringe]",
				@"🎉": @"[tada]",
				@"🎋": @"[tanabata_tree]",
				@"🍊": @"[tangerine]",
				@"🍵": @"[tea]",
				@"📞": @"[telephone_receiver]",
				@"🔭": @"[telescope]",
				@"🎾": @"[tennis]",
				@"🚽": @"[toilet]",
				@"🍅": @"[tomato]",
				@"🎩": @"[tophat]",
				@"📐": @"[triangular_ruler]",
				@"🏆": @"[trophy]",
				@"🍹": @"[tropical_drink]",
				@"🎺": @"[trumpet]",
				@"📺": @"[tv]",
				@"🔓": @"[unlock]",
				@"📼": @"[vhs]",
				@"📹": @"[video_camera]",
				@"🎮": @"[video_game]",
				@"🎻": @"[violin]",
				@"⌚": @"[watch]",
				@"🍉": @"[watermelon]",
				@"🍷": @"[wine_glass]",
				@"👚": @"[womans_clothes]",
				@"👒": @"[womans_hat]",
				@"🔧": @"[wrench]",
				@"💴": @"[yen]",
				@"🚡": @"[aerial_tramway]",
				@"✈": @"[airplane]",
				@"🚑": @"[ambulance]",
				@"⚓": @"[anchor]",
				@"🚛": @"[articulated_lorry]",
				@"🏧": @"[atm]",
				@"🏦": @"[bank]",
				@"💈": @"[barber]",
				@"🔰": @"[beginner]",
				@"🚲": @"[bike]",
				@"🚙": @"[blue_car]",
				@"⛵": @"[boat]",
				@"🌉": @"[bridge_at_night]",
				@"🚅": @"[bullettrain_front]",
				@"🚄": @"[bullettrain_side]",
				@"🚌": @"[bus]",
				@"🚏": @"[busstop]",
				@"🚗": @"[car]",
				@"🎠": @"[carousel_horse]",
				@"🏁": @"[checkered_flag]",
				@"⛪": @"[church]",
				@"🎪": @"[circus_tent]",
				@"🌇": @"[city_sunrise]",
				@"🌆": @"[city_sunset]",
				@"🚧": @"[construction]",
				@"🏪": @"[convenience_store]",
				@"🎌": @"[crossed_flags]",
				@"🏬": @"[department_store]",
				@"🏰": @"[european_castle]",
				@"🏤": @"[european_post_office]",
				@"🏭": @"[factory]",
				@"🎡": @"[ferris_wheel]",
				@"🚒": @"[fire_engine]",
				@"⛲": @"[fountain]",
				@"⛽": @"[fuelpump]",
				@"🚁": @"[helicopter]",
				@"🏥": @"[hospital]",
				@"🏨": @"[hotel]",
				@"♨": @"[hotsprings]",
				@"🏠": @"[house]",
				@"🏡": @"[house_with_garden]",
				@"🗾": @"[japan]",
				@"🏯": @"[japanese_castle]",
				@"🚈": @"[light_rail]",
				@"🏩": @"[love_hotel]",
				@"🚐": @"[minibus]",
				@"🚝": @"[monorail]",
				@"🗻": @"[mount_fuji]",
				@"🚠": @"[mountain_cableway]",
				@"🚞": @"[mountain_railway]",
				@"🗿": @"[moyai]",
				@"🏢": @"[office]",
				@"🚘": @"[oncoming_automobile]",
				@"🚍": @"[oncoming_bus]",
				@"🚔": @"[oncoming_police_car]",
				@"🚖": @"[oncoming_taxi]",
				@"🎭": @"[performing_arts]",
				@"🚓": @"[police_car]",
				@"🏣": @"[post_office]",
				@"🚃": @"[railway_car]",
				@"🌈": @"[rainbow]",
				@"🚀": @"[rocket]",
				@"🎢": @"[roller_coaster]",
				@"🚨": @"[rotating_light]",
				@"📍": @"[round_pushpin]",
				@"🚣": @"[rowboat]",
				@"🏫": @"[school]",
				@"🚢": @"[ship]",
				@"🎰": @"[slot_machine]",
				@"🚤": @"[speedboat]",
				@"🌃": @"[stars]",
				@"🚉": @"[station]",
				@"🗽": @"[statue_of_liberty]",
				@"🚂": @"[steam_locomotive]",
				@"🌅": @"[sunrise]",
				@"🌄": @"[sunrise_over_mountains]",
				@"🚟": @"[suspension_railway]",
				@"🚕": @"[taxi]",
				@"⛺": @"[tent]",
				@"🎫": @"[ticket]",
				@"🗼": @"[tokyo_tower]",
				@"🚜": @"[tractor]",
				@"🚥": @"[traffic_light]",
				@"🚆": @"[train2]",
				@"🚊": @"[tram]",
				@"🚩": @"[triangular_flag_on_post]",
				@"🚎": @"[trolleybus]",
				@"🚚": @"[truck]",
				@"🚦": @"[vertical_traffic_light]",
				@"⚠": @"[warning]",
				@"💒": @"[wedding]",
				@"🇯🇵": @"[jp]",
				@"🇰🇷": @"[kr]",
				@"🇨🇳": @"[cn]",
				@"🇺🇸": @"[us]",
				@"🇫🇷": @"[fr]",
				@"🇪🇸": @"[es]",
				@"🇮🇹": @"[it]",
				@"🇷🇺": @"[ru]",
				@"🇬🇧": @"[gb]",
				@"🇩🇪": @"[de]",
				@"💯": @"[100]",
				@"🔢": @"[1234]",
				@"🅰": @"[a]",
				@"🆎": @"[ab]",
				@"🔤": @"[abc]",
				@"🔡": @"[abcd]",
				@"🉑": @"[accept]",
				@"♒": @"[aquarius]",
				@"♈": @"[aries]",
				@"◀": @"[arrow_backward]",
				@"⏬": @"[arrow_double_down]",
				@"⏫": @"[arrow_double_up]",
				@"⬇": @"[arrow_down]",
				@"🔽": @"[arrow_down_small]",
				@"▶": @"[arrow_forward]",
				@"⤵": @"[arrow_heading_down]",
				@"⤴": @"[arrow_heading_up]",
				@"⬅": @"[arrow_left]",
				@"↙": @"[arrow_lower_left]",
				@"↘": @"[arrow_lower_right]",
				@"➡": @"[arrow_right]",
				@"↪": @"[arrow_right_hook]",
				@"⬆": @"[arrow_up]",
				@"↕": @"[arrow_up_down]",
				@"🔼": @"[arrow_up_small]",
				@"↖": @"[arrow_upper_left]",
				@"↗": @"[arrow_upper_right]",
				@"🔃": @"[arrows_clockwise]",
				@"🔄": @"[arrows_counterclockwise]",
				@"🅱": @"[b]",
				@"🚼": @"[baby_symbol]",
				@"🛄": @"[baggage_claim]",
				@"☑": @"[ballot_box_with_check]",
				@"‼": @"[bangbang]",
				@"⚫": @"[black_circle]",
				@"🔲": @"[black_square_button]",
				@"♋": @"[cancer]",
				@"🔠": @"[capital_abcd]",
				@"♑": @"[capricorn]",
				@"💹": @"[chart]",
				@"🚸": @"[children_crossing]",
				@"🎦": @"[cinema]",
				@"🆑": @"[cl]",
				@"🕐": @"[clock1]",
				@"🕙": @"[clock10]",
				@"🕥": @"[clock1030]",
				@"🕚": @"[clock11]",
				@"🕦": @"[clock1130]",
				@"🕛": @"[clock12]",
				@"🕧": @"[clock1230]",
				@"🕜": @"[clock130]",
				@"🕑": @"[clock2]",
				@"🕝": @"[clock230]",
				@"🕒": @"[clock3]",
				@"🕞": @"[clock330]",
				@"🕓": @"[clock4]",
				@"🕟": @"[clock430]",
				@"🕔": @"[clock5]",
				@"🕠": @"[clock530]",
				@"🕕": @"[clock6]",
				@"🕡": @"[clock630]",
				@"🕖": @"[clock7]",
				@"🕢": @"[clock730]",
				@"🕗": @"[clock8]",
				@"🕣": @"[clock830]",
				@"🕘": @"[clock9]",
				@"🕤": @"[clock930]",
				@"㊗": @"[congratulations]",
				@"🆒": @"[cool]",
				@"©": @"[copyright]",
				@"➰": @"[curly_loop]",
				@"💱": @"[currency_exchange]",
				@"🛃": @"[customs]",
				@"💠": @"[diamond_shape_with_a_dot_inside]",
				@"🚯": @"[do_not_litter]",
				@"8⃣": @"[eight]",
				@"✴": @"[eight_pointed_black_star]",
				@"✳": @"[eight_spoked_asterisk]",
				@"🔚": @"[end]",
				@"⏩": @"[fast_forward]",
				@"5⃣": @"[five]",
				@"4⃣": @"[four]",
				@"🆓": @"[free]",
				@"♊": @"[gemini]",
				@"#⃣": @"[hash]",
				@"💟": @"[heart_decoration]",
				@"✔": @"[heavy_check_mark]",
				@"➗": @"[heavy_division_sign]",
				@"💲": @"[heavy_dollar_sign]",
				@"➖": @"[heavy_minus_sign]",
				@"✖": @"[heavy_multiplication_x]",
				@"➕": @"[heavy_plus_sign]",
				@"🆔": @"[id]",
				@"🉐": @"[ideograph_advantage]",
				@"ℹ": @"[information_source]",
				@"⁉": @"[interrobang]",
				@"🔟": @"[keycap_ten]",
				@"🈁": @"[koko]",
				@"🔵": @"[large_blue_circle]",
				@"🔷": @"[large_blue_diamond]",
				@"🔶": @"[large_orange_diamond]",
				@"🛅": @"[left_luggage]",
				@"↔": @"[left_right_arrow]",
				@"↩": @"[leftwards_arrow_with_hook]",
				@"♌": @"[leo]",
				@"♎": @"[libra]",
				@"🔗": @"[link]",
				@"Ⓜ": @"[m]",
				@"🚹": @"[mens]",
				@"🚇": @"[metro]",
				@"📴": @"[mobile_phone_off]",
				@"❎": @"[negative_squared_cross_mark]",
				@"🆕": @"[new]",
				@"🆖": @"[ng]",
				@"9⃣": @"[nine]",
				@"🚳": @"[no_bicycles]",
				@"⛔": @"[no_entry]",
				@"🚫": @"[no_entry_sign]",
				@"📵": @"[no_mobile_phones]",
				@"🚷": @"[no_pedestrians]",
				@"🚭": @"[no_smoking]",
				@"🚱": @"[non-potable_water]",
				@"⭕": @"[o]",
				@"🅾": @"[o2]",
				@"🆗": @"[ok]",
				@"🔛": @"[on]",
				@"1⃣": @"[one]",
				@"⛎": @"[ophiuchus]",
				@"🅿": @"[parking]",
				@"〽": @"[part_alternation_mark]",
				@"🛂": @"[passport_control]",
				@"♓": @"[pisces]",
				@"🚰": @"[potable_water]",
				@"🚮": @"[put_litter_in_its_place]",
				@"🔘": @"[radio_button]",
				@"♻": @"[recycle]",
				@"🔴": @"[red_circle]",
				@"®": @"[registered]",
				@"🔁": @"[repeat]",
				@"🔂": @"[repeat_one]",
				@"🚻": @"[restroom]",
				@"⏪": @"[rewind]",
				@"🈂": @"[sa]",
				@"♐": @"[sagittarius]",
				@"♏": @"[scorpius]",
				@"㊙": @"[secret]",
				@"7⃣": @"[seven]",
				@"📶": @"[signal_strength]",
				@"6⃣": @"[six]",
				@"🔯": @"[six_pointed_star]",
				@"🔹": @"[small_blue_diamond]",
				@"🔸": @"[small_orange_diamond]",
				@"🔺": @"[small_red_triangle]",
				@"🔻": @"[small_red_triangle_down]",
				@"🔜": @"[soon]",
				@"🆘": @"[sos]",
				@"🔣": @"[symbols]",
				@"♉": @"[taurus]",
				@"3⃣": @"[three]",
				@"™": @"[tm]",
				@"🔝": @"[top]",
				@"🔱": @"[trident]",
				@"🔀": @"[twisted_rightwards_arrows]",
				@"2⃣": @"[two]",
				@"🈹": @"[u5272]",
				@"🈴": @"[u5408]",
				@"🈺": @"[u55b6]",
				@"🈯": @"[u6307]",
				@"🈷": @"[u6708]",
				@"🈶": @"[u6709]",
				@"🈵": @"[u6e80]",
				@"🈚": @"[u7121]",
				@"🈸": @"[u7533]",
				@"🈲": @"[u7981]",
				@"🈳": @"[u7a7a]",
				@"🔞": @"[underage]",
				@"🆙": @"[up]",
				@"📳": @"[vibration_mode]",
				@"♍": @"[virgo]",
				@"🆚": @"[vs]",
				@"〰": @"[wavy_dash]",
				@"🚾": @"[wc]",
				@"♿": @"[wheelchair]",
				@"✅": @"[white_check_mark]",
				@"⚪": @"[white_circle]",
				@"💮": @"[white_flower]",
				@"🔳": @"[white_square_button]",
				@"🚺": @"[womens]",
				@"❌": @"[x]",
				@"0⃣": @"[zero]"
				};
	
	__block NSString *text = self;
	[emojis enumerateKeysAndObjectsUsingBlock:^(id key, id obj, BOOL *stop) {
		text = [text stringByReplacingOccurrencesOfString:key withString:obj];
	}];
	
	return text;
}

@end
